#define _CFFI_

/* We try to define Py_LIMITED_API before including Python.h.

   Mess: we can only define it if Py_DEBUG, Py_TRACE_REFS and
   Py_REF_DEBUG are not defined.  This is a best-effort approximation:
   we can learn about Py_DEBUG from pyconfig.h, but it is unclear if
   the same works for the other two macros.  Py_DEBUG implies them,
   but not the other way around.

   The implementation is messy (issue #350): on Windows, with _MSC_VER,
   we have to define Py_LIMITED_API even before including pyconfig.h.
   In that case, we guess what pyconfig.h will do to the macros above,
   and check our guess after the #include.

   Note that on Windows, with CPython 3.x, you need >= 3.5 and virtualenv
   version >= 16.0.0.  With older versions of either, you don't get a
   copy of PYTHON3.DLL in the virtualenv.  We can't check the version of
   CPython *before* we even include pyconfig.h.  ffi.set_source() puts
   a ``#define _CFFI_NO_LIMITED_API'' at the start of this file if it is
   running on Windows < 3.5, as an attempt at fixing it, but that's
   arguably wrong because it may not be the target version of Python.
   Still better than nothing I guess.  As another workaround, you can
   remove the definition of Py_LIMITED_API here.

   See also 'py_limited_api' in cffi/setuptools_ext.py.
*/
#if !defined(_CFFI_USE_EMBEDDING) && !defined(Py_LIMITED_API)
#  ifdef _MSC_VER
#    if !defined(_DEBUG) && !defined(Py_DEBUG) && !defined(Py_TRACE_REFS) && !defined(Py_REF_DEBUG) && !defined(_CFFI_NO_LIMITED_API)
#      define Py_LIMITED_API
#    endif
#    include <pyconfig.h>
     /* sanity-check: Py_LIMITED_API will cause crashes if any of these
        are also defined.  Normally, the Python file PC/pyconfig.h does not
        cause any of these to be defined, with the exception that _DEBUG
        causes Py_DEBUG.  Double-check that. */
#    ifdef Py_LIMITED_API
#      if defined(Py_DEBUG)
#        error "pyconfig.h unexpectedly defines Py_DEBUG, but Py_LIMITED_API is set"
#      endif
#      if defined(Py_TRACE_REFS)
#        error "pyconfig.h unexpectedly defines Py_TRACE_REFS, but Py_LIMITED_API is set"
#      endif
#      if defined(Py_REF_DEBUG)
#        error "pyconfig.h unexpectedly defines Py_REF_DEBUG, but Py_LIMITED_API is set"
#      endif
#    endif
#  else
#    include <pyconfig.h>
#    if !defined(Py_DEBUG) && !defined(Py_TRACE_REFS) && !defined(Py_REF_DEBUG) && !defined(_CFFI_NO_LIMITED_API)
#      define Py_LIMITED_API
#    endif
#  endif
#endif

#include <Python.h>
#ifdef __cplusplus
extern "C" {
#endif
#include <stddef.h>

/* This part is from file 'cffi/parse_c_type.h'.  It is copied at the
   beginning of C sources generated by CFFI's ffi.set_source(). */

typedef void *_cffi_opcode_t;

#define _CFFI_OP(opcode, arg)   (_cffi_opcode_t)(opcode | (((uintptr_t)(arg)) << 8))
#define _CFFI_GETOP(cffi_opcode)    ((unsigned char)(uintptr_t)cffi_opcode)
#define _CFFI_GETARG(cffi_opcode)   (((intptr_t)cffi_opcode) >> 8)

#define _CFFI_OP_PRIMITIVE       1
#define _CFFI_OP_POINTER         3
#define _CFFI_OP_ARRAY           5
#define _CFFI_OP_OPEN_ARRAY      7
#define _CFFI_OP_STRUCT_UNION    9
#define _CFFI_OP_ENUM           11
#define _CFFI_OP_FUNCTION       13
#define _CFFI_OP_FUNCTION_END   15
#define _CFFI_OP_NOOP           17
#define _CFFI_OP_BITFIELD       19
#define _CFFI_OP_TYPENAME       21
#define _CFFI_OP_CPYTHON_BLTN_V 23   // varargs
#define _CFFI_OP_CPYTHON_BLTN_N 25   // noargs
#define _CFFI_OP_CPYTHON_BLTN_O 27   // O  (i.e. a single arg)
#define _CFFI_OP_CONSTANT       29
#define _CFFI_OP_CONSTANT_INT   31
#define _CFFI_OP_GLOBAL_VAR     33
#define _CFFI_OP_DLOPEN_FUNC    35
#define _CFFI_OP_DLOPEN_CONST   37
#define _CFFI_OP_GLOBAL_VAR_F   39
#define _CFFI_OP_EXTERN_PYTHON  41

#define _CFFI_PRIM_VOID          0
#define _CFFI_PRIM_BOOL          1
#define _CFFI_PRIM_CHAR          2
#define _CFFI_PRIM_SCHAR         3
#define _CFFI_PRIM_UCHAR         4
#define _CFFI_PRIM_SHORT         5
#define _CFFI_PRIM_USHORT        6
#define _CFFI_PRIM_INT           7
#define _CFFI_PRIM_UINT          8
#define _CFFI_PRIM_LONG          9
#define _CFFI_PRIM_ULONG        10
#define _CFFI_PRIM_LONGLONG     11
#define _CFFI_PRIM_ULONGLONG    12
#define _CFFI_PRIM_FLOAT        13
#define _CFFI_PRIM_DOUBLE       14
#define _CFFI_PRIM_LONGDOUBLE   15

#define _CFFI_PRIM_WCHAR        16
#define _CFFI_PRIM_INT8         17
#define _CFFI_PRIM_UINT8        18
#define _CFFI_PRIM_INT16        19
#define _CFFI_PRIM_UINT16       20
#define _CFFI_PRIM_INT32        21
#define _CFFI_PRIM_UINT32       22
#define _CFFI_PRIM_INT64        23
#define _CFFI_PRIM_UINT64       24
#define _CFFI_PRIM_INTPTR       25
#define _CFFI_PRIM_UINTPTR      26
#define _CFFI_PRIM_PTRDIFF      27
#define _CFFI_PRIM_SIZE         28
#define _CFFI_PRIM_SSIZE        29
#define _CFFI_PRIM_INT_LEAST8   30
#define _CFFI_PRIM_UINT_LEAST8  31
#define _CFFI_PRIM_INT_LEAST16  32
#define _CFFI_PRIM_UINT_LEAST16 33
#define _CFFI_PRIM_INT_LEAST32  34
#define _CFFI_PRIM_UINT_LEAST32 35
#define _CFFI_PRIM_INT_LEAST64  36
#define _CFFI_PRIM_UINT_LEAST64 37
#define _CFFI_PRIM_INT_FAST8    38
#define _CFFI_PRIM_UINT_FAST8   39
#define _CFFI_PRIM_INT_FAST16   40
#define _CFFI_PRIM_UINT_FAST16  41
#define _CFFI_PRIM_INT_FAST32   42
#define _CFFI_PRIM_UINT_FAST32  43
#define _CFFI_PRIM_INT_FAST64   44
#define _CFFI_PRIM_UINT_FAST64  45
#define _CFFI_PRIM_INTMAX       46
#define _CFFI_PRIM_UINTMAX      47
#define _CFFI_PRIM_FLOATCOMPLEX 48
#define _CFFI_PRIM_DOUBLECOMPLEX 49
#define _CFFI_PRIM_CHAR16       50
#define _CFFI_PRIM_CHAR32       51

#define _CFFI__NUM_PRIM         52
#define _CFFI__UNKNOWN_PRIM           (-1)
#define _CFFI__UNKNOWN_FLOAT_PRIM     (-2)
#define _CFFI__UNKNOWN_LONG_DOUBLE    (-3)

#define _CFFI__IO_FILE_STRUCT         (-1)


struct _cffi_global_s {
    const char *name;
    void *address;
    _cffi_opcode_t type_op;
    void *size_or_direct_fn;  // OP_GLOBAL_VAR: size, or 0 if unknown
                              // OP_CPYTHON_BLTN_*: addr of direct function
};

struct _cffi_getconst_s {
    unsigned long long value;
    const struct _cffi_type_context_s *ctx;
    int gindex;
};

struct _cffi_struct_union_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_STRUCT_UNION
    int flags;               // _CFFI_F_* flags below
    size_t size;
    int alignment;
    int first_field_index;   // -> _cffi_fields array
    int num_fields;
};
#define _CFFI_F_UNION         0x01   // is a union, not a struct
#define _CFFI_F_CHECK_FIELDS  0x02   // complain if fields are not in the
                                     // "standard layout" or if some are missing
#define _CFFI_F_PACKED        0x04   // for CHECK_FIELDS, assume a packed struct
#define _CFFI_F_EXTERNAL      0x08   // in some other ffi.include()
#define _CFFI_F_OPAQUE        0x10   // opaque

struct _cffi_field_s {
    const char *name;
    size_t field_offset;
    size_t field_size;
    _cffi_opcode_t field_type_op;
};

struct _cffi_enum_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_ENUM
    int type_prim;           // _CFFI_PRIM_xxx
    const char *enumerators; // comma-delimited string
};

struct _cffi_typename_s {
    const char *name;
    int type_index;   /* if opaque, points to a possibly artificial
                         OP_STRUCT which is itself opaque */
};

struct _cffi_type_context_s {
    _cffi_opcode_t *types;
    const struct _cffi_global_s *globals;
    const struct _cffi_field_s *fields;
    const struct _cffi_struct_union_s *struct_unions;
    const struct _cffi_enum_s *enums;
    const struct _cffi_typename_s *typenames;
    int num_globals;
    int num_struct_unions;
    int num_enums;
    int num_typenames;
    const char *const *includes;
    int num_types;
    int flags;      /* future extension */
};

struct _cffi_parse_info_s {
    const struct _cffi_type_context_s *ctx;
    _cffi_opcode_t *output;
    unsigned int output_size;
    size_t error_location;
    const char *error_message;
};

struct _cffi_externpy_s {
    const char *name;
    size_t size_of_result;
    void *reserved1, *reserved2;
};

#ifdef _CFFI_INTERNAL
static int parse_c_type(struct _cffi_parse_info_s *info, const char *input);
static int search_in_globals(const struct _cffi_type_context_s *ctx,
                             const char *search, size_t search_len);
static int search_in_struct_unions(const struct _cffi_type_context_s *ctx,
                                   const char *search, size_t search_len);
#endif

/* this block of #ifs should be kept exactly identical between
   c/_cffi_backend.c, cffi/vengine_cpy.py, cffi/vengine_gen.py
   and cffi/_cffi_include.h */
#if defined(_MSC_VER)
# include <malloc.h>   /* for alloca() */
# if _MSC_VER < 1600   /* MSVC < 2010 */
   typedef __int8 int8_t;
   typedef __int16 int16_t;
   typedef __int32 int32_t;
   typedef __int64 int64_t;
   typedef unsigned __int8 uint8_t;
   typedef unsigned __int16 uint16_t;
   typedef unsigned __int32 uint32_t;
   typedef unsigned __int64 uint64_t;
   typedef __int8 int_least8_t;
   typedef __int16 int_least16_t;
   typedef __int32 int_least32_t;
   typedef __int64 int_least64_t;
   typedef unsigned __int8 uint_least8_t;
   typedef unsigned __int16 uint_least16_t;
   typedef unsigned __int32 uint_least32_t;
   typedef unsigned __int64 uint_least64_t;
   typedef __int8 int_fast8_t;
   typedef __int16 int_fast16_t;
   typedef __int32 int_fast32_t;
   typedef __int64 int_fast64_t;
   typedef unsigned __int8 uint_fast8_t;
   typedef unsigned __int16 uint_fast16_t;
   typedef unsigned __int32 uint_fast32_t;
   typedef unsigned __int64 uint_fast64_t;
   typedef __int64 intmax_t;
   typedef unsigned __int64 uintmax_t;
# else
#  include <stdint.h>
# endif
# if _MSC_VER < 1800   /* MSVC < 2013 */
#  ifndef __cplusplus
    typedef unsigned char _Bool;
#  endif
# endif
#else
# include <stdint.h>
# if (defined (__SVR4) && defined (__sun)) || defined(_AIX) || defined(__hpux)
#  include <alloca.h>
# endif
#endif

#ifdef __GNUC__
# define _CFFI_UNUSED_FN  __attribute__((unused))
#else
# define _CFFI_UNUSED_FN  /* nothing */
#endif

#ifdef __cplusplus
# ifndef _Bool
   typedef bool _Bool;   /* semi-hackish: C++ has no _Bool; bool is builtin */
# endif
#endif

/**********  CPython-specific section  **********/
#ifndef PYPY_VERSION


#if PY_MAJOR_VERSION >= 3
# define PyInt_FromLong PyLong_FromLong
#endif

#define _cffi_from_c_double PyFloat_FromDouble
#define _cffi_from_c_float PyFloat_FromDouble
#define _cffi_from_c_long PyInt_FromLong
#define _cffi_from_c_ulong PyLong_FromUnsignedLong
#define _cffi_from_c_longlong PyLong_FromLongLong
#define _cffi_from_c_ulonglong PyLong_FromUnsignedLongLong
#define _cffi_from_c__Bool PyBool_FromLong

#define _cffi_to_c_double PyFloat_AsDouble
#define _cffi_to_c_float PyFloat_AsDouble

#define _cffi_from_c_int(x, type)                                        \
    (((type)-1) > 0 ? /* unsigned */                                     \
        (sizeof(type) < sizeof(long) ?                                   \
            PyInt_FromLong((long)x) :                                    \
         sizeof(type) == sizeof(long) ?                                  \
            PyLong_FromUnsignedLong((unsigned long)x) :                  \
            PyLong_FromUnsignedLongLong((unsigned long long)x)) :        \
        (sizeof(type) <= sizeof(long) ?                                  \
            PyInt_FromLong((long)x) :                                    \
            PyLong_FromLongLong((long long)x)))

#define _cffi_to_c_int(o, type)                                          \
    ((type)(                                                             \
     sizeof(type) == 1 ? (((type)-1) > 0 ? (type)_cffi_to_c_u8(o)        \
                                         : (type)_cffi_to_c_i8(o)) :     \
     sizeof(type) == 2 ? (((type)-1) > 0 ? (type)_cffi_to_c_u16(o)       \
                                         : (type)_cffi_to_c_i16(o)) :    \
     sizeof(type) == 4 ? (((type)-1) > 0 ? (type)_cffi_to_c_u32(o)       \
                                         : (type)_cffi_to_c_i32(o)) :    \
     sizeof(type) == 8 ? (((type)-1) > 0 ? (type)_cffi_to_c_u64(o)       \
                                         : (type)_cffi_to_c_i64(o)) :    \
     (Py_FatalError("unsupported size for type " #type), (type)0)))

#define _cffi_to_c_i8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[1])
#define _cffi_to_c_u8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[2])
#define _cffi_to_c_i16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[3])
#define _cffi_to_c_u16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[4])
#define _cffi_to_c_i32                                                   \
                 ((int(*)(PyObject *))_cffi_exports[5])
#define _cffi_to_c_u32                                                   \
                 ((unsigned int(*)(PyObject *))_cffi_exports[6])
#define _cffi_to_c_i64                                                   \
                 ((long long(*)(PyObject *))_cffi_exports[7])
#define _cffi_to_c_u64                                                   \
                 ((unsigned long long(*)(PyObject *))_cffi_exports[8])
#define _cffi_to_c_char                                                  \
                 ((int(*)(PyObject *))_cffi_exports[9])
#define _cffi_from_c_pointer                                             \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[10])
#define _cffi_to_c_pointer                                               \
    ((char *(*)(PyObject *, struct _cffi_ctypedescr *))_cffi_exports[11])
#define _cffi_get_struct_layout                                          \
    not used any more
#define _cffi_restore_errno                                              \
    ((void(*)(void))_cffi_exports[13])
#define _cffi_save_errno                                                 \
    ((void(*)(void))_cffi_exports[14])
#define _cffi_from_c_char                                                \
    ((PyObject *(*)(char))_cffi_exports[15])
#define _cffi_from_c_deref                                               \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[16])
#define _cffi_to_c                                                       \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[17])
#define _cffi_from_c_struct                                              \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[18])
#define _cffi_to_c_wchar_t                                               \
    ((_cffi_wchar_t(*)(PyObject *))_cffi_exports[19])
#define _cffi_from_c_wchar_t                                             \
    ((PyObject *(*)(_cffi_wchar_t))_cffi_exports[20])
#define _cffi_to_c_long_double                                           \
    ((long double(*)(PyObject *))_cffi_exports[21])
#define _cffi_to_c__Bool                                                 \
    ((_Bool(*)(PyObject *))_cffi_exports[22])
#define _cffi_prepare_pointer_call_argument                              \
    ((Py_ssize_t(*)(struct _cffi_ctypedescr *,                           \
                    PyObject *, char **))_cffi_exports[23])
#define _cffi_convert_array_from_object                                  \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[24])
#define _CFFI_CPIDX  25
#define _cffi_call_python                                                \
    ((void(*)(struct _cffi_externpy_s *, char *))_cffi_exports[_CFFI_CPIDX])
#define _cffi_to_c_wchar3216_t                                           \
    ((int(*)(PyObject *))_cffi_exports[26])
#define _cffi_from_c_wchar3216_t                                         \
    ((PyObject *(*)(int))_cffi_exports[27])
#define _CFFI_NUM_EXPORTS 28

struct _cffi_ctypedescr;

static void *_cffi_exports[_CFFI_NUM_EXPORTS];

#define _cffi_type(index)   (                           \
    assert((((uintptr_t)_cffi_types[index]) & 1) == 0), \
    (struct _cffi_ctypedescr *)_cffi_types[index])

static PyObject *_cffi_init(const char *module_name, Py_ssize_t version,
                            const struct _cffi_type_context_s *ctx)
{
    PyObject *module, *o_arg, *new_module;
    void *raw[] = {
        (void *)module_name,
        (void *)version,
        (void *)_cffi_exports,
        (void *)ctx,
    };

    module = PyImport_ImportModule("_cffi_backend");
    if (module == NULL)
        goto failure;

    o_arg = PyLong_FromVoidPtr((void *)raw);
    if (o_arg == NULL)
        goto failure;

    new_module = PyObject_CallMethod(
        module, (char *)"_init_cffi_1_0_external_module", (char *)"O", o_arg);

    Py_DECREF(o_arg);
    Py_DECREF(module);
    return new_module;

  failure:
    Py_XDECREF(module);
    return NULL;
}


#ifdef HAVE_WCHAR_H
typedef wchar_t _cffi_wchar_t;
#else
typedef uint16_t _cffi_wchar_t;   /* same random pick as _cffi_backend.c */
#endif

_CFFI_UNUSED_FN static uint16_t _cffi_to_c_char16_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return (uint16_t)_cffi_to_c_wchar_t(o);
    else
        return (uint16_t)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char16_t(uint16_t x)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

_CFFI_UNUSED_FN static int _cffi_to_c_char32_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return (int)_cffi_to_c_wchar_t(o);
    else
        return (int)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char32_t(unsigned int x)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

union _cffi_union_alignment_u {
    unsigned char m_char;
    unsigned short m_short;
    unsigned int m_int;
    unsigned long m_long;
    unsigned long long m_longlong;
    float m_float;
    double m_double;
    long double m_longdouble;
};

struct _cffi_freeme_s {
    struct _cffi_freeme_s *next;
    union _cffi_union_alignment_u alignment;
};

_CFFI_UNUSED_FN static int
_cffi_convert_array_argument(struct _cffi_ctypedescr *ctptr, PyObject *arg,
                             char **output_data, Py_ssize_t datasize,
                             struct _cffi_freeme_s **freeme)
{
    char *p;
    if (datasize < 0)
        return -1;

    p = *output_data;
    if (p == NULL) {
        struct _cffi_freeme_s *fp = (struct _cffi_freeme_s *)PyObject_Malloc(
            offsetof(struct _cffi_freeme_s, alignment) + (size_t)datasize);
        if (fp == NULL)
            return -1;
        fp->next = *freeme;
        *freeme = fp;
        p = *output_data = (char *)&fp->alignment;
    }
    memset((void *)p, 0, (size_t)datasize);
    return _cffi_convert_array_from_object(p, ctptr, arg);
}

_CFFI_UNUSED_FN static void
_cffi_free_array_arguments(struct _cffi_freeme_s *freeme)
{
    do {
        void *p = (void *)freeme;
        freeme = freeme->next;
        PyObject_Free(p);
    } while (freeme != NULL);
}

/**********  end CPython-specific section  **********/
#else
_CFFI_UNUSED_FN
static void (*_cffi_call_python_org)(struct _cffi_externpy_s *, char *);
# define _cffi_call_python  _cffi_call_python_org
#endif


#define _cffi_array_len(array)   (sizeof(array) / sizeof((array)[0]))

#define _cffi_prim_int(size, sign)                                      \
    ((size) == 1 ? ((sign) ? _CFFI_PRIM_INT8  : _CFFI_PRIM_UINT8)  :    \
     (size) == 2 ? ((sign) ? _CFFI_PRIM_INT16 : _CFFI_PRIM_UINT16) :    \
     (size) == 4 ? ((sign) ? _CFFI_PRIM_INT32 : _CFFI_PRIM_UINT32) :    \
     (size) == 8 ? ((sign) ? _CFFI_PRIM_INT64 : _CFFI_PRIM_UINT64) :    \
     _CFFI__UNKNOWN_PRIM)

#define _cffi_prim_float(size)                                          \
    ((size) == sizeof(float) ? _CFFI_PRIM_FLOAT :                       \
     (size) == sizeof(double) ? _CFFI_PRIM_DOUBLE :                     \
     (size) == sizeof(long double) ? _CFFI__UNKNOWN_LONG_DOUBLE :       \
     _CFFI__UNKNOWN_FLOAT_PRIM)

#define _cffi_check_int(got, got_nonpos, expected)      \
    ((got_nonpos) == (expected <= 0) &&                 \
     (got) == (unsigned long long)expected)

#ifdef MS_WIN32
# define _cffi_stdcall  __stdcall
#else
# define _cffi_stdcall  /* nothing */
#endif

#ifdef __cplusplus
}
#endif

/************************************************************/


    #include "bgen/bgen.h"
    #include <stdbool.h>
#include <stddef.h>
#include <stdint.h>
#include <stdlib.h>

static void read_ploidy(struct bgen_genotype const* genotype, uint8_t* ploidy,
                        uint32_t nsamples)
{
    for (uint32_t i = 0; i < nsamples; ++i)
        ploidy[i] = bgen_genotype_ploidy(genotype, i);
}

static void read_missing(struct bgen_genotype const* genotype, bool* missing,
                         uint32_t nsamples)
{
    for (uint32_t i = 0; i < nsamples; ++i)
        missing[i] = bgen_genotype_missing(genotype, i);
}

    #include <stddef.h>
#include <stdint.h>
#include <stdlib.h>

#ifndef MAX
#define MAX(X, Y) ((X) > (Y) ? (X) : (Y))
#endif

static void read_partition_part1(struct bgen_partition const* partition, uint32_t* position,
                                 uint16_t* nalleles, uint64_t* offset, uint32_t* id_max_len,
                                 uint32_t* rsid_max_len, uint32_t* chrom_max_len,
                                 uint32_t* allele_ids_max_len)
{
    uint32_t nvariants = bgen_partition_nvariants(partition);

    *id_max_len = 0;
    *rsid_max_len = 0;
    *chrom_max_len = 0;
    *allele_ids_max_len = 0;

    for (uint32_t i = 0; i < nvariants; ++i) {
        struct bgen_variant const* v = bgen_partition_get_variant(partition, i);
        position[i] = v->position;
        nalleles[i] = v->nalleles;
        offset[i] = v->genotype_offset;

        *id_max_len = MAX(*id_max_len, (uint32_t)bgen_string_length(v->id));
        *rsid_max_len = MAX(*rsid_max_len, (uint32_t)bgen_string_length(v->rsid));
        *chrom_max_len = MAX(*chrom_max_len, (uint32_t)bgen_string_length(v->chrom));

        if (v->nalleles == 0)
            continue;

        uint32_t length = v->nalleles - 1;
        for (uint16_t j = 0; j < v->nalleles; ++j)
            length += (uint32_t)bgen_string_length(v->allele_ids[j]);
        *allele_ids_max_len = MAX(*allele_ids_max_len, length);
    }
}

static void read_partition_part2(struct bgen_partition const* partition, char* const id,
                                 uint32_t id_stride, char* const rsid, uint32_t rsid_stride,
                                 char* const chrom, uint32_t chrom_stride,
                                 char* const allele_ids, uint32_t allele_ids_stride)
{
    uint32_t nvariants = bgen_partition_nvariants(partition);
    for (uint32_t i = 0; i < nvariants; ++i) {
        struct bgen_variant const* v = bgen_partition_get_variant(partition, i);

        memcpy(id + i * id_stride, bgen_string_data(v->id), bgen_string_length(v->id));

        memcpy(rsid + i * rsid_stride, bgen_string_data(v->rsid), bgen_string_length(v->rsid));

        memcpy(chrom + i * chrom_stride, bgen_string_data(v->chrom),
               bgen_string_length(v->chrom));

        size_t j = 0;
        for (uint16_t r = 0; r < v->nalleles; ++r) {

            memcpy(allele_ids + i * allele_ids_stride + j, bgen_string_data(v->allele_ids[r]),
                   bgen_string_length(v->allele_ids[r]));
            j += bgen_string_length(v->allele_ids[r]);

            if (r + 1 < v->nalleles) {
                memcpy(allele_ids + i * allele_ids_stride + j, ",", 1);
                j += 1;
            }
        }
    }
}

    #include <stddef.h>
#include <stdint.h>
#include <stdlib.h>

#ifndef MAX
#define MAX(X, Y) ((X) > (Y) ? (X) : (Y))
#endif

static void read_samples_part1(struct bgen_samples* samples, uint32_t nsamples,
                               uint32_t* samples_max_len)
{
    *samples_max_len = 0;

    for (uint32_t i = 0; i < nsamples; ++i) {
        struct bgen_string const* sample = bgen_samples_get(samples, i);
        *samples_max_len = MAX(*samples_max_len, (uint32_t)bgen_string_length(sample));
    }
}

static void read_samples_part2(struct bgen_samples const* samples, uint32_t nsamples,
                               char* const sample_array, uint32_t samples_stride)
{
    for (uint32_t i = 0; i < nsamples; ++i) {
        struct bgen_string const* sample = bgen_samples_get(samples, i);
        memcpy(sample_array + i * samples_stride, bgen_string_data(sample),
               bgen_string_length(sample));
    }
}

    

/************************************************************/

static void *_cffi_types[] = {
/*  0 */ _CFFI_OP(_CFFI_OP_FUNCTION, 145), // _Bool()(struct bgen_file const *)
/*  1 */ _CFFI_OP(_CFFI_OP_POINTER, 150), // struct bgen_file const *
/*  2 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/*  3 */ _CFFI_OP(_CFFI_OP_FUNCTION, 145), // _Bool()(struct bgen_genotype const *)
/*  4 */ _CFFI_OP(_CFFI_OP_POINTER, 151), // struct bgen_genotype const *
/*  5 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/*  6 */ _CFFI_OP(_CFFI_OP_FUNCTION, 145), // _Bool()(struct bgen_genotype const *, uint32_t)
/*  7 */ _CFFI_OP(_CFFI_OP_NOOP, 4),
/*  8 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 22), // uint32_t
/*  9 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 10 */ _CFFI_OP(_CFFI_OP_FUNCTION, 145), // _Bool()(struct bgen_string, struct bgen_string)
/* 11 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 5), // struct bgen_string
/* 12 */ _CFFI_OP(_CFFI_OP_NOOP, 11),
/* 13 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 14 */ _CFFI_OP(_CFFI_OP_FUNCTION, 35), // char const *()(struct bgen_string const *)
/* 15 */ _CFFI_OP(_CFFI_OP_POINTER, 11), // struct bgen_string const *
/* 16 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 17 */ _CFFI_OP(_CFFI_OP_FUNCTION, 48), // int()(struct bgen_genotype *, double *)
/* 18 */ _CFFI_OP(_CFFI_OP_POINTER, 151), // struct bgen_genotype *
/* 19 */ _CFFI_OP(_CFFI_OP_POINTER, 147), // double *
/* 20 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 21 */ _CFFI_OP(_CFFI_OP_FUNCTION, 48), // int()(struct bgen_genotype *, float *)
/* 22 */ _CFFI_OP(_CFFI_OP_NOOP, 18),
/* 23 */ _CFFI_OP(_CFFI_OP_POINTER, 148), // float *
/* 24 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 25 */ _CFFI_OP(_CFFI_OP_FUNCTION, 48), // int()(struct bgen_metafile const *)
/* 26 */ _CFFI_OP(_CFFI_OP_POINTER, 153), // struct bgen_metafile const *
/* 27 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 28 */ _CFFI_OP(_CFFI_OP_FUNCTION, 149), // int32_t()(struct bgen_file const *)
/* 29 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 30 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 31 */ _CFFI_OP(_CFFI_OP_FUNCTION, 59), // size_t()(struct bgen_string const *)
/* 32 */ _CFFI_OP(_CFFI_OP_NOOP, 15),
/* 33 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 34 */ _CFFI_OP(_CFFI_OP_FUNCTION, 38), // struct bgen_file *()(char const *)
/* 35 */ _CFFI_OP(_CFFI_OP_POINTER, 146), // char const *
/* 36 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 37 */ _CFFI_OP(_CFFI_OP_FUNCTION, 18), // struct bgen_genotype *()(struct bgen_file *, uint64_t)
/* 38 */ _CFFI_OP(_CFFI_OP_POINTER, 150), // struct bgen_file *
/* 39 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 24), // uint64_t
/* 40 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 41 */ _CFFI_OP(_CFFI_OP_FUNCTION, 152), // struct bgen_metafile *()(char const *)
/* 42 */ _CFFI_OP(_CFFI_OP_NOOP, 35),
/* 43 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 44 */ _CFFI_OP(_CFFI_OP_FUNCTION, 152), // struct bgen_metafile *()(struct bgen_file *, char const *, uint32_t, int)
/* 45 */ _CFFI_OP(_CFFI_OP_NOOP, 38),
/* 46 */ _CFFI_OP(_CFFI_OP_NOOP, 35),
/* 47 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 22),
/* 48 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7), // int
/* 49 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 50 */ _CFFI_OP(_CFFI_OP_FUNCTION, 66), // struct bgen_partition const *()(struct bgen_metafile const *, uint32_t)
/* 51 */ _CFFI_OP(_CFFI_OP_NOOP, 26),
/* 52 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 22),
/* 53 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 54 */ _CFFI_OP(_CFFI_OP_FUNCTION, 129), // struct bgen_samples *()(struct bgen_file *)
/* 55 */ _CFFI_OP(_CFFI_OP_NOOP, 38),
/* 56 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 57 */ _CFFI_OP(_CFFI_OP_FUNCTION, 15), // struct bgen_string const *()(char const *, size_t)
/* 58 */ _CFFI_OP(_CFFI_OP_NOOP, 35),
/* 59 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28), // size_t
/* 60 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 61 */ _CFFI_OP(_CFFI_OP_FUNCTION, 15), // struct bgen_string const *()(struct bgen_samples const *, uint32_t)
/* 62 */ _CFFI_OP(_CFFI_OP_POINTER, 155), // struct bgen_samples const *
/* 63 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 22),
/* 64 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 65 */ _CFFI_OP(_CFFI_OP_FUNCTION, 157), // struct bgen_variant const *()(struct bgen_partition const *, uint32_t)
/* 66 */ _CFFI_OP(_CFFI_OP_POINTER, 154), // struct bgen_partition const *
/* 67 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 22),
/* 68 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 69 */ _CFFI_OP(_CFFI_OP_FUNCTION, 159), // uint16_t()(struct bgen_genotype const *)
/* 70 */ _CFFI_OP(_CFFI_OP_NOOP, 4),
/* 71 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 72 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // uint32_t()(struct bgen_metafile const *)
/* 73 */ _CFFI_OP(_CFFI_OP_NOOP, 26),
/* 74 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 75 */ _CFFI_OP(_CFFI_OP_FUNCTION, 8), // uint32_t()(struct bgen_partition const *)
/* 76 */ _CFFI_OP(_CFFI_OP_NOOP, 66),
/* 77 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 78 */ _CFFI_OP(_CFFI_OP_FUNCTION, 160), // uint8_t()(struct bgen_genotype const *)
/* 79 */ _CFFI_OP(_CFFI_OP_NOOP, 4),
/* 80 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 81 */ _CFFI_OP(_CFFI_OP_FUNCTION, 160), // uint8_t()(struct bgen_genotype const *, uint32_t)
/* 82 */ _CFFI_OP(_CFFI_OP_NOOP, 4),
/* 83 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 22),
/* 84 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 85 */ _CFFI_OP(_CFFI_OP_FUNCTION, 161), // unsigned int()(struct bgen_genotype const *)
/* 86 */ _CFFI_OP(_CFFI_OP_NOOP, 4),
/* 87 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 88 */ _CFFI_OP(_CFFI_OP_FUNCTION, 162), // void()(struct bgen_file const *)
/* 89 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 90 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 91 */ _CFFI_OP(_CFFI_OP_FUNCTION, 162), // void()(struct bgen_genotype const *)
/* 92 */ _CFFI_OP(_CFFI_OP_NOOP, 4),
/* 93 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 94 */ _CFFI_OP(_CFFI_OP_FUNCTION, 162), // void()(struct bgen_genotype const *, _Bool *, uint32_t)
/* 95 */ _CFFI_OP(_CFFI_OP_NOOP, 4),
/* 96 */ _CFFI_OP(_CFFI_OP_POINTER, 145), // _Bool *
/* 97 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 22),
/* 98 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 99 */ _CFFI_OP(_CFFI_OP_FUNCTION, 162), // void()(struct bgen_genotype const *, uint8_t *, uint32_t)
/* 100 */ _CFFI_OP(_CFFI_OP_NOOP, 4),
/* 101 */ _CFFI_OP(_CFFI_OP_POINTER, 160), // uint8_t *
/* 102 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 22),
/* 103 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 104 */ _CFFI_OP(_CFFI_OP_FUNCTION, 162), // void()(struct bgen_partition const *)
/* 105 */ _CFFI_OP(_CFFI_OP_NOOP, 66),
/* 106 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 107 */ _CFFI_OP(_CFFI_OP_FUNCTION, 162), // void()(struct bgen_partition const *, char *, uint32_t, char *, uint32_t, char *, uint32_t, char *, uint32_t)
/* 108 */ _CFFI_OP(_CFFI_OP_NOOP, 66),
/* 109 */ _CFFI_OP(_CFFI_OP_POINTER, 146), // char *
/* 110 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 22),
/* 111 */ _CFFI_OP(_CFFI_OP_NOOP, 109),
/* 112 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 22),
/* 113 */ _CFFI_OP(_CFFI_OP_NOOP, 109),
/* 114 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 22),
/* 115 */ _CFFI_OP(_CFFI_OP_NOOP, 109),
/* 116 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 22),
/* 117 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 118 */ _CFFI_OP(_CFFI_OP_FUNCTION, 162), // void()(struct bgen_partition const *, uint32_t *, uint16_t *, uint64_t *, uint32_t *, uint32_t *, uint32_t *, uint32_t *)
/* 119 */ _CFFI_OP(_CFFI_OP_NOOP, 66),
/* 120 */ _CFFI_OP(_CFFI_OP_POINTER, 8), // uint32_t *
/* 121 */ _CFFI_OP(_CFFI_OP_POINTER, 159), // uint16_t *
/* 122 */ _CFFI_OP(_CFFI_OP_POINTER, 39), // uint64_t *
/* 123 */ _CFFI_OP(_CFFI_OP_NOOP, 120),
/* 124 */ _CFFI_OP(_CFFI_OP_NOOP, 120),
/* 125 */ _CFFI_OP(_CFFI_OP_NOOP, 120),
/* 126 */ _CFFI_OP(_CFFI_OP_NOOP, 120),
/* 127 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 128 */ _CFFI_OP(_CFFI_OP_FUNCTION, 162), // void()(struct bgen_samples *, uint32_t, uint32_t *)
/* 129 */ _CFFI_OP(_CFFI_OP_POINTER, 155), // struct bgen_samples *
/* 130 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 22),
/* 131 */ _CFFI_OP(_CFFI_OP_NOOP, 120),
/* 132 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 133 */ _CFFI_OP(_CFFI_OP_FUNCTION, 162), // void()(struct bgen_samples const *)
/* 134 */ _CFFI_OP(_CFFI_OP_NOOP, 62),
/* 135 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 136 */ _CFFI_OP(_CFFI_OP_FUNCTION, 162), // void()(struct bgen_samples const *, uint32_t, char *, uint32_t)
/* 137 */ _CFFI_OP(_CFFI_OP_NOOP, 62),
/* 138 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 22),
/* 139 */ _CFFI_OP(_CFFI_OP_NOOP, 109),
/* 140 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 22),
/* 141 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 142 */ _CFFI_OP(_CFFI_OP_FUNCTION, 162), // void()(struct bgen_string const *)
/* 143 */ _CFFI_OP(_CFFI_OP_NOOP, 15),
/* 144 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 145 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 1), // _Bool
/* 146 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 2), // char
/* 147 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 14), // double
/* 148 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13), // float
/* 149 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 21), // int32_t
/* 150 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 0), // struct bgen_file
/* 151 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 1), // struct bgen_genotype
/* 152 */ _CFFI_OP(_CFFI_OP_POINTER, 153), // struct bgen_metafile *
/* 153 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 2), // struct bgen_metafile
/* 154 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 3), // struct bgen_partition
/* 155 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 4), // struct bgen_samples
/* 156 */ _CFFI_OP(_CFFI_OP_POINTER, 15), // struct bgen_string const * *
/* 157 */ _CFFI_OP(_CFFI_OP_POINTER, 158), // struct bgen_variant const *
/* 158 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 6), // struct bgen_variant
/* 159 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 20), // uint16_t
/* 160 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 18), // uint8_t
/* 161 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 8), // unsigned int
/* 162 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 0), // void
};

static void _cffi_d_bgen_file_close(struct bgen_file const * x0)
{
  bgen_file_close(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_bgen_file_close(PyObject *self, PyObject *arg0)
{
  struct bgen_file const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct bgen_file const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { bgen_file_close(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_bgen_file_close _cffi_d_bgen_file_close
#endif

static _Bool _cffi_d_bgen_file_contain_samples(struct bgen_file const * x0)
{
  return bgen_file_contain_samples(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_bgen_file_contain_samples(PyObject *self, PyObject *arg0)
{
  struct bgen_file const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  _Bool result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct bgen_file const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = bgen_file_contain_samples(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c__Bool(result);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_bgen_file_contain_samples _cffi_d_bgen_file_contain_samples
#endif

static int32_t _cffi_d_bgen_file_nsamples(struct bgen_file const * x0)
{
  return bgen_file_nsamples(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_bgen_file_nsamples(PyObject *self, PyObject *arg0)
{
  struct bgen_file const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int32_t result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct bgen_file const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = bgen_file_nsamples(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int32_t);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_bgen_file_nsamples _cffi_d_bgen_file_nsamples
#endif

static int32_t _cffi_d_bgen_file_nvariants(struct bgen_file const * x0)
{
  return bgen_file_nvariants(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_bgen_file_nvariants(PyObject *self, PyObject *arg0)
{
  struct bgen_file const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int32_t result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct bgen_file const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = bgen_file_nvariants(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int32_t);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_bgen_file_nvariants _cffi_d_bgen_file_nvariants
#endif

static struct bgen_file * _cffi_d_bgen_file_open(char const * x0)
{
  return bgen_file_open(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_bgen_file_open(PyObject *self, PyObject *arg0)
{
  char const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct bgen_file * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(35), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(35), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = bgen_file_open(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(38));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_bgen_file_open _cffi_d_bgen_file_open
#endif

static struct bgen_genotype * _cffi_d_bgen_file_open_genotype(struct bgen_file * x0, uint64_t x1)
{
  return bgen_file_open_genotype(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_bgen_file_open_genotype(PyObject *self, PyObject *args)
{
  struct bgen_file * x0;
  uint64_t x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct bgen_genotype * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "bgen_file_open_genotype", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(38), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct bgen_file *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(38), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, uint64_t);
  if (x1 == (uint64_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = bgen_file_open_genotype(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(18));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_bgen_file_open_genotype _cffi_d_bgen_file_open_genotype
#endif

static struct bgen_samples * _cffi_d_bgen_file_read_samples(struct bgen_file * x0)
{
  return bgen_file_read_samples(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_bgen_file_read_samples(PyObject *self, PyObject *arg0)
{
  struct bgen_file * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct bgen_samples * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(38), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct bgen_file *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(38), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = bgen_file_read_samples(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(129));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_bgen_file_read_samples _cffi_d_bgen_file_read_samples
#endif

static void _cffi_d_bgen_genotype_close(struct bgen_genotype const * x0)
{
  bgen_genotype_close(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_bgen_genotype_close(PyObject *self, PyObject *arg0)
{
  struct bgen_genotype const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct bgen_genotype const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { bgen_genotype_close(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_bgen_genotype_close _cffi_d_bgen_genotype_close
#endif

static uint8_t _cffi_d_bgen_genotype_max_ploidy(struct bgen_genotype const * x0)
{
  return bgen_genotype_max_ploidy(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_bgen_genotype_max_ploidy(PyObject *self, PyObject *arg0)
{
  struct bgen_genotype const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  uint8_t result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct bgen_genotype const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = bgen_genotype_max_ploidy(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, uint8_t);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_bgen_genotype_max_ploidy _cffi_d_bgen_genotype_max_ploidy
#endif

static uint8_t _cffi_d_bgen_genotype_min_ploidy(struct bgen_genotype const * x0)
{
  return bgen_genotype_min_ploidy(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_bgen_genotype_min_ploidy(PyObject *self, PyObject *arg0)
{
  struct bgen_genotype const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  uint8_t result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct bgen_genotype const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = bgen_genotype_min_ploidy(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, uint8_t);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_bgen_genotype_min_ploidy _cffi_d_bgen_genotype_min_ploidy
#endif

static _Bool _cffi_d_bgen_genotype_missing(struct bgen_genotype const * x0, uint32_t x1)
{
  return bgen_genotype_missing(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_bgen_genotype_missing(PyObject *self, PyObject *args)
{
  struct bgen_genotype const * x0;
  uint32_t x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  _Bool result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "bgen_genotype_missing", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct bgen_genotype const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, uint32_t);
  if (x1 == (uint32_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = bgen_genotype_missing(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c__Bool(result);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_bgen_genotype_missing _cffi_d_bgen_genotype_missing
#endif

static uint16_t _cffi_d_bgen_genotype_nalleles(struct bgen_genotype const * x0)
{
  return bgen_genotype_nalleles(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_bgen_genotype_nalleles(PyObject *self, PyObject *arg0)
{
  struct bgen_genotype const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  uint16_t result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct bgen_genotype const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = bgen_genotype_nalleles(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, uint16_t);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_bgen_genotype_nalleles _cffi_d_bgen_genotype_nalleles
#endif

static unsigned int _cffi_d_bgen_genotype_ncombs(struct bgen_genotype const * x0)
{
  return bgen_genotype_ncombs(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_bgen_genotype_ncombs(PyObject *self, PyObject *arg0)
{
  struct bgen_genotype const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  unsigned int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct bgen_genotype const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = bgen_genotype_ncombs(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, unsigned int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_bgen_genotype_ncombs _cffi_d_bgen_genotype_ncombs
#endif

static _Bool _cffi_d_bgen_genotype_phased(struct bgen_genotype const * x0)
{
  return bgen_genotype_phased(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_bgen_genotype_phased(PyObject *self, PyObject *arg0)
{
  struct bgen_genotype const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  _Bool result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct bgen_genotype const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = bgen_genotype_phased(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c__Bool(result);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_bgen_genotype_phased _cffi_d_bgen_genotype_phased
#endif

static uint8_t _cffi_d_bgen_genotype_ploidy(struct bgen_genotype const * x0, uint32_t x1)
{
  return bgen_genotype_ploidy(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_bgen_genotype_ploidy(PyObject *self, PyObject *args)
{
  struct bgen_genotype const * x0;
  uint32_t x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  uint8_t result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "bgen_genotype_ploidy", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct bgen_genotype const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, uint32_t);
  if (x1 == (uint32_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = bgen_genotype_ploidy(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, uint8_t);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_bgen_genotype_ploidy _cffi_d_bgen_genotype_ploidy
#endif

static int _cffi_d_bgen_genotype_read(struct bgen_genotype * x0, double * x1)
{
  return bgen_genotype_read(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_bgen_genotype_read(PyObject *self, PyObject *args)
{
  struct bgen_genotype * x0;
  double * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "bgen_genotype_read", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(18), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct bgen_genotype *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(18), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(19), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(19), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = bgen_genotype_read(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_bgen_genotype_read _cffi_d_bgen_genotype_read
#endif

static int _cffi_d_bgen_genotype_read32(struct bgen_genotype * x0, float * x1)
{
  return bgen_genotype_read32(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_bgen_genotype_read32(PyObject *self, PyObject *args)
{
  struct bgen_genotype * x0;
  float * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "bgen_genotype_read32", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(18), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct bgen_genotype *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(18), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(23), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (float *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(23), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = bgen_genotype_read32(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_bgen_genotype_read32 _cffi_d_bgen_genotype_read32
#endif

static int _cffi_d_bgen_genotype_read64(struct bgen_genotype * x0, double * x1)
{
  return bgen_genotype_read64(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_bgen_genotype_read64(PyObject *self, PyObject *args)
{
  struct bgen_genotype * x0;
  double * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "bgen_genotype_read64", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(18), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct bgen_genotype *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(18), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(19), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(19), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = bgen_genotype_read64(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_bgen_genotype_read64 _cffi_d_bgen_genotype_read64
#endif

static int _cffi_d_bgen_metafile_close(struct bgen_metafile const * x0)
{
  return bgen_metafile_close(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_bgen_metafile_close(PyObject *self, PyObject *arg0)
{
  struct bgen_metafile const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(26), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct bgen_metafile const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(26), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = bgen_metafile_close(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_bgen_metafile_close _cffi_d_bgen_metafile_close
#endif

static struct bgen_metafile * _cffi_d_bgen_metafile_create(struct bgen_file * x0, char const * x1, uint32_t x2, int x3)
{
  return bgen_metafile_create(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_bgen_metafile_create(PyObject *self, PyObject *args)
{
  struct bgen_file * x0;
  char const * x1;
  uint32_t x2;
  int x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct bgen_metafile * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "bgen_metafile_create", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(38), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct bgen_file *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(38), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(35), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(35), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, uint32_t);
  if (x2 == (uint32_t)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = bgen_metafile_create(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(152));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_bgen_metafile_create _cffi_d_bgen_metafile_create
#endif

static uint32_t _cffi_d_bgen_metafile_npartitions(struct bgen_metafile const * x0)
{
  return bgen_metafile_npartitions(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_bgen_metafile_npartitions(PyObject *self, PyObject *arg0)
{
  struct bgen_metafile const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  uint32_t result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(26), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct bgen_metafile const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(26), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = bgen_metafile_npartitions(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, uint32_t);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_bgen_metafile_npartitions _cffi_d_bgen_metafile_npartitions
#endif

static uint32_t _cffi_d_bgen_metafile_nvariants(struct bgen_metafile const * x0)
{
  return bgen_metafile_nvariants(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_bgen_metafile_nvariants(PyObject *self, PyObject *arg0)
{
  struct bgen_metafile const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  uint32_t result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(26), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct bgen_metafile const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(26), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = bgen_metafile_nvariants(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, uint32_t);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_bgen_metafile_nvariants _cffi_d_bgen_metafile_nvariants
#endif

static struct bgen_metafile * _cffi_d_bgen_metafile_open(char const * x0)
{
  return bgen_metafile_open(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_bgen_metafile_open(PyObject *self, PyObject *arg0)
{
  char const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct bgen_metafile * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(35), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(35), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = bgen_metafile_open(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(152));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_bgen_metafile_open _cffi_d_bgen_metafile_open
#endif

static struct bgen_partition const * _cffi_d_bgen_metafile_read_partition(struct bgen_metafile const * x0, uint32_t x1)
{
  return bgen_metafile_read_partition(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_bgen_metafile_read_partition(PyObject *self, PyObject *args)
{
  struct bgen_metafile const * x0;
  uint32_t x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct bgen_partition const * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "bgen_metafile_read_partition", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(26), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct bgen_metafile const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(26), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, uint32_t);
  if (x1 == (uint32_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = bgen_metafile_read_partition(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(66));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_bgen_metafile_read_partition _cffi_d_bgen_metafile_read_partition
#endif

static void _cffi_d_bgen_partition_destroy(struct bgen_partition const * x0)
{
  bgen_partition_destroy(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_bgen_partition_destroy(PyObject *self, PyObject *arg0)
{
  struct bgen_partition const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(66), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct bgen_partition const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(66), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { bgen_partition_destroy(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_bgen_partition_destroy _cffi_d_bgen_partition_destroy
#endif

static struct bgen_variant const * _cffi_d_bgen_partition_get_variant(struct bgen_partition const * x0, uint32_t x1)
{
  return bgen_partition_get_variant(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_bgen_partition_get_variant(PyObject *self, PyObject *args)
{
  struct bgen_partition const * x0;
  uint32_t x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct bgen_variant const * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "bgen_partition_get_variant", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(66), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct bgen_partition const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(66), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, uint32_t);
  if (x1 == (uint32_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = bgen_partition_get_variant(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(157));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_bgen_partition_get_variant _cffi_d_bgen_partition_get_variant
#endif

static uint32_t _cffi_d_bgen_partition_nvariants(struct bgen_partition const * x0)
{
  return bgen_partition_nvariants(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_bgen_partition_nvariants(PyObject *self, PyObject *arg0)
{
  struct bgen_partition const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  uint32_t result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(66), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct bgen_partition const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(66), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = bgen_partition_nvariants(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, uint32_t);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_bgen_partition_nvariants _cffi_d_bgen_partition_nvariants
#endif

static void _cffi_d_bgen_samples_destroy(struct bgen_samples const * x0)
{
  bgen_samples_destroy(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_bgen_samples_destroy(PyObject *self, PyObject *arg0)
{
  struct bgen_samples const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(62), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct bgen_samples const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(62), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { bgen_samples_destroy(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_bgen_samples_destroy _cffi_d_bgen_samples_destroy
#endif

static struct bgen_string const * _cffi_d_bgen_samples_get(struct bgen_samples const * x0, uint32_t x1)
{
  return bgen_samples_get(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_bgen_samples_get(PyObject *self, PyObject *args)
{
  struct bgen_samples const * x0;
  uint32_t x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct bgen_string const * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "bgen_samples_get", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(62), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct bgen_samples const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(62), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, uint32_t);
  if (x1 == (uint32_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = bgen_samples_get(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(15));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_bgen_samples_get _cffi_d_bgen_samples_get
#endif

static struct bgen_string const * _cffi_d_bgen_string_create(char const * x0, size_t x1)
{
  return bgen_string_create(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_bgen_string_create(PyObject *self, PyObject *args)
{
  char const * x0;
  size_t x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  struct bgen_string const * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "bgen_string_create", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(35), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(35), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, size_t);
  if (x1 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = bgen_string_create(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(15));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_bgen_string_create _cffi_d_bgen_string_create
#endif

static char const * _cffi_d_bgen_string_data(struct bgen_string const * x0)
{
  return bgen_string_data(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_bgen_string_data(PyObject *self, PyObject *arg0)
{
  struct bgen_string const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char const * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(15), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct bgen_string const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(15), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = bgen_string_data(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(35));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_bgen_string_data _cffi_d_bgen_string_data
#endif

static void _cffi_d_bgen_string_destroy(struct bgen_string const * x0)
{
  bgen_string_destroy(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_bgen_string_destroy(PyObject *self, PyObject *arg0)
{
  struct bgen_string const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(15), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct bgen_string const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(15), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { bgen_string_destroy(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_bgen_string_destroy _cffi_d_bgen_string_destroy
#endif

static _Bool _cffi_d_bgen_string_equal(struct bgen_string x0, struct bgen_string x1)
{
  return bgen_string_equal(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_bgen_string_equal(PyObject *self, PyObject *args)
{
  struct bgen_string x0;
  struct bgen_string x1;
  _Bool result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "bgen_string_equal", 2, 2, &arg0, &arg1))
    return NULL;

  if (_cffi_to_c((char *)&x0, _cffi_type(11), arg0) < 0)
    return NULL;

  if (_cffi_to_c((char *)&x1, _cffi_type(11), arg1) < 0)
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = bgen_string_equal(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c__Bool(result);
  return pyresult;
}
#else
static _Bool _cffi_f_bgen_string_equal(struct bgen_string *x0, struct bgen_string *x1)
{
  _Bool result;
  { result = bgen_string_equal(*x0, *x1); }
  return result;
}
#endif

static size_t _cffi_d_bgen_string_length(struct bgen_string const * x0)
{
  return bgen_string_length(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_bgen_string_length(PyObject *self, PyObject *arg0)
{
  struct bgen_string const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  size_t result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(15), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct bgen_string const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(15), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = bgen_string_length(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, size_t);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_bgen_string_length _cffi_d_bgen_string_length
#endif

static void _cffi_d_read_missing(struct bgen_genotype const * x0, _Bool * x1, uint32_t x2)
{
  read_missing(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_read_missing(PyObject *self, PyObject *args)
{
  struct bgen_genotype const * x0;
  _Bool * x1;
  uint32_t x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "read_missing", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct bgen_genotype const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(96), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (_Bool *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(96), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, uint32_t);
  if (x2 == (uint32_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { read_missing(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_read_missing _cffi_d_read_missing
#endif

static void _cffi_d_read_partition_part1(struct bgen_partition const * x0, uint32_t * x1, uint16_t * x2, uint64_t * x3, uint32_t * x4, uint32_t * x5, uint32_t * x6, uint32_t * x7)
{
  read_partition_part1(x0, x1, x2, x3, x4, x5, x6, x7);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_read_partition_part1(PyObject *self, PyObject *args)
{
  struct bgen_partition const * x0;
  uint32_t * x1;
  uint16_t * x2;
  uint64_t * x3;
  uint32_t * x4;
  uint32_t * x5;
  uint32_t * x6;
  uint32_t * x7;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;
  PyObject *arg6;
  PyObject *arg7;

  if (!PyArg_UnpackTuple(args, "read_partition_part1", 8, 8, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5, &arg6, &arg7))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(66), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct bgen_partition const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(66), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(120), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (uint32_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(120), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(121), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (uint16_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(121), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(122), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (uint64_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(122), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(120), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (uint32_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(120), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(120), arg5, (char **)&x5);
  if (datasize != 0) {
    x5 = ((size_t)datasize) <= 640 ? (uint32_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(120), arg5, (char **)&x5,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(120), arg6, (char **)&x6);
  if (datasize != 0) {
    x6 = ((size_t)datasize) <= 640 ? (uint32_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(120), arg6, (char **)&x6,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(120), arg7, (char **)&x7);
  if (datasize != 0) {
    x7 = ((size_t)datasize) <= 640 ? (uint32_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(120), arg7, (char **)&x7,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { read_partition_part1(x0, x1, x2, x3, x4, x5, x6, x7); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_read_partition_part1 _cffi_d_read_partition_part1
#endif

static void _cffi_d_read_partition_part2(struct bgen_partition const * x0, char * x1, uint32_t x2, char * x3, uint32_t x4, char * x5, uint32_t x6, char * x7, uint32_t x8)
{
  read_partition_part2(x0, x1, x2, x3, x4, x5, x6, x7, x8);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_read_partition_part2(PyObject *self, PyObject *args)
{
  struct bgen_partition const * x0;
  char * x1;
  uint32_t x2;
  char * x3;
  uint32_t x4;
  char * x5;
  uint32_t x6;
  char * x7;
  uint32_t x8;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;
  PyObject *arg5;
  PyObject *arg6;
  PyObject *arg7;
  PyObject *arg8;

  if (!PyArg_UnpackTuple(args, "read_partition_part2", 9, 9, &arg0, &arg1, &arg2, &arg3, &arg4, &arg5, &arg6, &arg7, &arg8))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(66), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct bgen_partition const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(66), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(109), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(109), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, uint32_t);
  if (x2 == (uint32_t)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(109), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(109), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x4 = _cffi_to_c_int(arg4, uint32_t);
  if (x4 == (uint32_t)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(109), arg5, (char **)&x5);
  if (datasize != 0) {
    x5 = ((size_t)datasize) <= 640 ? (char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(109), arg5, (char **)&x5,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x6 = _cffi_to_c_int(arg6, uint32_t);
  if (x6 == (uint32_t)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(109), arg7, (char **)&x7);
  if (datasize != 0) {
    x7 = ((size_t)datasize) <= 640 ? (char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(109), arg7, (char **)&x7,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x8 = _cffi_to_c_int(arg8, uint32_t);
  if (x8 == (uint32_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { read_partition_part2(x0, x1, x2, x3, x4, x5, x6, x7, x8); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_read_partition_part2 _cffi_d_read_partition_part2
#endif

static void _cffi_d_read_ploidy(struct bgen_genotype const * x0, uint8_t * x1, uint32_t x2)
{
  read_ploidy(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_read_ploidy(PyObject *self, PyObject *args)
{
  struct bgen_genotype const * x0;
  uint8_t * x1;
  uint32_t x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "read_ploidy", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct bgen_genotype const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(101), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (uint8_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(101), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, uint32_t);
  if (x2 == (uint32_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { read_ploidy(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_read_ploidy _cffi_d_read_ploidy
#endif

static void _cffi_d_read_samples_part1(struct bgen_samples * x0, uint32_t x1, uint32_t * x2)
{
  read_samples_part1(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_read_samples_part1(PyObject *self, PyObject *args)
{
  struct bgen_samples * x0;
  uint32_t x1;
  uint32_t * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "read_samples_part1", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(129), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct bgen_samples *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(129), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, uint32_t);
  if (x1 == (uint32_t)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(120), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (uint32_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(120), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { read_samples_part1(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_read_samples_part1 _cffi_d_read_samples_part1
#endif

static void _cffi_d_read_samples_part2(struct bgen_samples const * x0, uint32_t x1, char * x2, uint32_t x3)
{
  read_samples_part2(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_read_samples_part2(PyObject *self, PyObject *args)
{
  struct bgen_samples const * x0;
  uint32_t x1;
  char * x2;
  uint32_t x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "read_samples_part2", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(62), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (struct bgen_samples const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(62), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, uint32_t);
  if (x1 == (uint32_t)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(109), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(109), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x3 = _cffi_to_c_int(arg3, uint32_t);
  if (x3 == (uint32_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { read_samples_part2(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_read_samples_part2 _cffi_d_read_samples_part2
#endif

_CFFI_UNUSED_FN
static void _cffi_checkfld_struct_bgen_variant(struct bgen_variant *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  (void)((p->genotype_offset) | 0);  /* check that 'struct bgen_variant.genotype_offset' is an integer */
  { struct bgen_string const * *tmp = &p->id; (void)tmp; }
  { struct bgen_string const * *tmp = &p->rsid; (void)tmp; }
  { struct bgen_string const * *tmp = &p->chrom; (void)tmp; }
  (void)((p->position) | 0);  /* check that 'struct bgen_variant.position' is an integer */
  (void)((p->nalleles) | 0);  /* check that 'struct bgen_variant.nalleles' is an integer */
  { struct bgen_string const * * *tmp = &p->allele_ids; (void)tmp; }
}
struct _cffi_align_struct_bgen_variant { char x; struct bgen_variant y; };

static const struct _cffi_global_s _cffi_globals[] = {
  { "bgen_file_close", (void *)_cffi_f_bgen_file_close, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 88), (void *)_cffi_d_bgen_file_close },
  { "bgen_file_contain_samples", (void *)_cffi_f_bgen_file_contain_samples, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 0), (void *)_cffi_d_bgen_file_contain_samples },
  { "bgen_file_nsamples", (void *)_cffi_f_bgen_file_nsamples, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 28), (void *)_cffi_d_bgen_file_nsamples },
  { "bgen_file_nvariants", (void *)_cffi_f_bgen_file_nvariants, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 28), (void *)_cffi_d_bgen_file_nvariants },
  { "bgen_file_open", (void *)_cffi_f_bgen_file_open, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 34), (void *)_cffi_d_bgen_file_open },
  { "bgen_file_open_genotype", (void *)_cffi_f_bgen_file_open_genotype, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 37), (void *)_cffi_d_bgen_file_open_genotype },
  { "bgen_file_read_samples", (void *)_cffi_f_bgen_file_read_samples, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 54), (void *)_cffi_d_bgen_file_read_samples },
  { "bgen_genotype_close", (void *)_cffi_f_bgen_genotype_close, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 91), (void *)_cffi_d_bgen_genotype_close },
  { "bgen_genotype_max_ploidy", (void *)_cffi_f_bgen_genotype_max_ploidy, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 78), (void *)_cffi_d_bgen_genotype_max_ploidy },
  { "bgen_genotype_min_ploidy", (void *)_cffi_f_bgen_genotype_min_ploidy, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 78), (void *)_cffi_d_bgen_genotype_min_ploidy },
  { "bgen_genotype_missing", (void *)_cffi_f_bgen_genotype_missing, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 6), (void *)_cffi_d_bgen_genotype_missing },
  { "bgen_genotype_nalleles", (void *)_cffi_f_bgen_genotype_nalleles, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 69), (void *)_cffi_d_bgen_genotype_nalleles },
  { "bgen_genotype_ncombs", (void *)_cffi_f_bgen_genotype_ncombs, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 85), (void *)_cffi_d_bgen_genotype_ncombs },
  { "bgen_genotype_phased", (void *)_cffi_f_bgen_genotype_phased, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 3), (void *)_cffi_d_bgen_genotype_phased },
  { "bgen_genotype_ploidy", (void *)_cffi_f_bgen_genotype_ploidy, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 81), (void *)_cffi_d_bgen_genotype_ploidy },
  { "bgen_genotype_read", (void *)_cffi_f_bgen_genotype_read, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 17), (void *)_cffi_d_bgen_genotype_read },
  { "bgen_genotype_read32", (void *)_cffi_f_bgen_genotype_read32, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 21), (void *)_cffi_d_bgen_genotype_read32 },
  { "bgen_genotype_read64", (void *)_cffi_f_bgen_genotype_read64, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 17), (void *)_cffi_d_bgen_genotype_read64 },
  { "bgen_metafile_close", (void *)_cffi_f_bgen_metafile_close, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 25), (void *)_cffi_d_bgen_metafile_close },
  { "bgen_metafile_create", (void *)_cffi_f_bgen_metafile_create, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 44), (void *)_cffi_d_bgen_metafile_create },
  { "bgen_metafile_npartitions", (void *)_cffi_f_bgen_metafile_npartitions, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 72), (void *)_cffi_d_bgen_metafile_npartitions },
  { "bgen_metafile_nvariants", (void *)_cffi_f_bgen_metafile_nvariants, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 72), (void *)_cffi_d_bgen_metafile_nvariants },
  { "bgen_metafile_open", (void *)_cffi_f_bgen_metafile_open, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 41), (void *)_cffi_d_bgen_metafile_open },
  { "bgen_metafile_read_partition", (void *)_cffi_f_bgen_metafile_read_partition, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 50), (void *)_cffi_d_bgen_metafile_read_partition },
  { "bgen_partition_destroy", (void *)_cffi_f_bgen_partition_destroy, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 104), (void *)_cffi_d_bgen_partition_destroy },
  { "bgen_partition_get_variant", (void *)_cffi_f_bgen_partition_get_variant, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 65), (void *)_cffi_d_bgen_partition_get_variant },
  { "bgen_partition_nvariants", (void *)_cffi_f_bgen_partition_nvariants, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 75), (void *)_cffi_d_bgen_partition_nvariants },
  { "bgen_samples_destroy", (void *)_cffi_f_bgen_samples_destroy, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 133), (void *)_cffi_d_bgen_samples_destroy },
  { "bgen_samples_get", (void *)_cffi_f_bgen_samples_get, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 61), (void *)_cffi_d_bgen_samples_get },
  { "bgen_string_create", (void *)_cffi_f_bgen_string_create, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 57), (void *)_cffi_d_bgen_string_create },
  { "bgen_string_data", (void *)_cffi_f_bgen_string_data, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 14), (void *)_cffi_d_bgen_string_data },
  { "bgen_string_destroy", (void *)_cffi_f_bgen_string_destroy, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 142), (void *)_cffi_d_bgen_string_destroy },
  { "bgen_string_equal", (void *)_cffi_f_bgen_string_equal, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 10), (void *)_cffi_d_bgen_string_equal },
  { "bgen_string_length", (void *)_cffi_f_bgen_string_length, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 31), (void *)_cffi_d_bgen_string_length },
  { "read_missing", (void *)_cffi_f_read_missing, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 94), (void *)_cffi_d_read_missing },
  { "read_partition_part1", (void *)_cffi_f_read_partition_part1, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 118), (void *)_cffi_d_read_partition_part1 },
  { "read_partition_part2", (void *)_cffi_f_read_partition_part2, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 107), (void *)_cffi_d_read_partition_part2 },
  { "read_ploidy", (void *)_cffi_f_read_ploidy, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 99), (void *)_cffi_d_read_ploidy },
  { "read_samples_part1", (void *)_cffi_f_read_samples_part1, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 128), (void *)_cffi_d_read_samples_part1 },
  { "read_samples_part2", (void *)_cffi_f_read_samples_part2, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 136), (void *)_cffi_d_read_samples_part2 },
};

static const struct _cffi_field_s _cffi_fields[] = {
  { "genotype_offset", offsetof(struct bgen_variant, genotype_offset),
                       sizeof(((struct bgen_variant *)0)->genotype_offset),
                       _CFFI_OP(_CFFI_OP_NOOP, 39) },
  { "id", offsetof(struct bgen_variant, id),
          sizeof(((struct bgen_variant *)0)->id),
          _CFFI_OP(_CFFI_OP_NOOP, 15) },
  { "rsid", offsetof(struct bgen_variant, rsid),
            sizeof(((struct bgen_variant *)0)->rsid),
            _CFFI_OP(_CFFI_OP_NOOP, 15) },
  { "chrom", offsetof(struct bgen_variant, chrom),
             sizeof(((struct bgen_variant *)0)->chrom),
             _CFFI_OP(_CFFI_OP_NOOP, 15) },
  { "position", offsetof(struct bgen_variant, position),
                sizeof(((struct bgen_variant *)0)->position),
                _CFFI_OP(_CFFI_OP_NOOP, 8) },
  { "nalleles", offsetof(struct bgen_variant, nalleles),
                sizeof(((struct bgen_variant *)0)->nalleles),
                _CFFI_OP(_CFFI_OP_NOOP, 159) },
  { "allele_ids", offsetof(struct bgen_variant, allele_ids),
                  sizeof(((struct bgen_variant *)0)->allele_ids),
                  _CFFI_OP(_CFFI_OP_NOOP, 156) },
};

static const struct _cffi_struct_union_s _cffi_struct_unions[] = {
  { "bgen_file", 150, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "bgen_genotype", 151, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "bgen_metafile", 153, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "bgen_partition", 154, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "bgen_samples", 155, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "bgen_string", 11, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "bgen_variant", 158, _CFFI_F_CHECK_FIELDS,
    sizeof(struct bgen_variant), offsetof(struct _cffi_align_struct_bgen_variant, y), 0, 7 },
};

static const struct _cffi_type_context_s _cffi_type_context = {
  _cffi_types,
  _cffi_globals,
  _cffi_fields,
  _cffi_struct_unions,
  NULL,  /* no enums */
  NULL,  /* no typenames */
  40,  /* num_globals */
  7,  /* num_struct_unions */
  0,  /* num_enums */
  0,  /* num_typenames */
  NULL,  /* no includes */
  163,  /* num_types */
  0,  /* flags */
};

#ifdef __GNUC__
#  pragma GCC visibility push(default)  /* for -fvisibility= */
#endif

#ifdef PYPY_VERSION
PyMODINIT_FUNC
_cffi_pypyinit__ffi(const void *p[])
{
    p[0] = (const void *)0x2601;
    p[1] = &_cffi_type_context;
#if PY_MAJOR_VERSION >= 3
    return NULL;
#endif
}
#  ifdef _MSC_VER
     PyMODINIT_FUNC
#  if PY_MAJOR_VERSION >= 3
     PyInit__ffi(void) { return NULL; }
#  else
     init_ffi(void) { }
#  endif
#  endif
#elif PY_MAJOR_VERSION >= 3
PyMODINIT_FUNC
PyInit__ffi(void)
{
  return _cffi_init("cbgen._ffi", 0x2601, &_cffi_type_context);
}
#else
PyMODINIT_FUNC
init_ffi(void)
{
  _cffi_init("cbgen._ffi", 0x2601, &_cffi_type_context);
}
#endif

#ifdef __GNUC__
#  pragma GCC visibility pop
#endif
